/**
 * Copyright (c) 2024 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "conversation.h"
#include "assert/assert.h"

conversation_t CONVERSATION;

void conversationInit() {
  memset(&CONVERSATION, 0, sizeof(conversation_t));
}

void conversationSet(
  const conversationcallback_t init,
  const conversationcallback_t update,
  void *user
) {
  CONVERSATION.init = init;
  CONVERSATION.update = update;
  CONVERSATION.index = CONVERSATION_NOT_INITIALIZED;
  CONVERSATION.user = user;
}

void conversationUpdate() {
  if(CONVERSATION.init == NULL || CONVERSATION.update == NULL) return;

  uint16_t ret;

  // Init or update.
  switch(CONVERSATION.index) {
    case CONVERSATION_NOT_INITIALIZED:
      CONVERSATION.index = 0;
      ret = CONVERSATION.init(
        &CONVERSATION, CONVERSATION.index
      );
      break;
    
    default:
      ret = CONVERSATION.update(&CONVERSATION, CONVERSATION.index);
      break;
  }

  // Check ret value and update conversation.
  switch(ret) {
    case CONVERSATION_NEXT:
      CONVERSATION.index++;
      CONVERSATION.index = CONVERSATION.init(
        &CONVERSATION, CONVERSATION.index
      );
      break;
    
    case CONVERSATION_CONTINUE:
      return;
    
    case CONVERSATION_RESTART:
      CONVERSATION.index = 0;
      CONVERSATION.init(&CONVERSATION, CONVERSATION.index);
      break;
    
    case CONVERSATION_INVALID:
      assertUnreachable("Invalid converstaion retval");
      break;

    case CONVERSATION_DONE:
      CONVERSATION.init = NULL;
      CONVERSATION.update = NULL;
      break;
    
    default:
      CONVERSATION.index = ret;
      break;
  }
}