/**
 * Copyright (c) 2024 Dominic Masters
 * 
 * This software is released under the MIT License.
 * https://opensource.org/licenses/MIT
 */

#include "conversationinteract.h"
#include "assert/assert.h"
#include "rpg/entity/entity.h"
#include "locale/language.h"
#include "ui/textbox.h"

uint16_t conversationInteractInit(conversation_t *convo, const uint16_t i) {
  assertNotNull(convo, "Conversation is NULL!");
  assertNotNull(convo->user, "Conversation user is NULL!");

  entity_t *e = (entity_t*)convo->user;
  switch(e->type) {
    case ENTITY_TYPE_SIGN:
      textboxSetText("entities.sign.name", e->sign.text);
      return 0;

    case ENTITY_TYPE_NPC:
      textboxSetText(e->npc.name, e->npc.text);
      return 0;

    default:
      assertUnreachable("Invalid entity type for conversation.");
      return CONVERSATION_INVALID;
  }
}

uint16_t conversationInteractUpdate(conversation_t *convo, const uint16_t i) {
  switch(i) {
    case 0:
      return textboxIsOpen() ? CONVERSATION_CONTINUE : CONVERSATION_DONE;
    
    default:
      return CONVERSATION_INVALID;
  }
}